# Aura Engine Features

This page gives a more detailed overview of the Aura Engine features and how to configure them.

<!-- TOC -->

- [Aura Engine Features](#aura-engine-features)
    - [Multi-channel Input (Filesystem, Stream, Analog)](#multi-channel-input-filesystem-stream-analog)
    - [Multi-channel output](#multi-channel-output)
        - [Analog line-out](#analog-line-out)
        - [Stream to Icecast](#stream-to-icecast)
        - [Record output to filesystem](#record-output-to-filesystem)
    - [Scheduling](#scheduling)
    - [Fallback Handling](#fallback-handling)
        - [Pro-active Fallback Handling (1st Level Fallback)](#pro-active-fallback-handling-1st-level-fallback)
        - [Fallback Handling using the Silence Detector (2nd Level Fallback)](#fallback-handling-using-the-silence-detector-2nd-level-fallback)
    - [Monitoring](#monitoring)
        - [Send mails on errors and warnings](#send-mails-on-errors-and-warnings)
        - [Engine Health Information via Engine API](#engine-health-information-via-engine-api)
        - [Engine Heartbeat](#engine-heartbeat)
        - [Logging](#logging)
    - [Read more](#read-more)

<!-- /TOC -->

## Multi-channel Input (Filesystem, Stream, Analog)

It's possible to schedules playlists with music or pre-recorded show stored on the **filessystem**,
via external **streams** or live from an **analog input** in the studio. All types of sources can
be mixed in a single playlist.

> Note: Since live sources and streams do not specify a length property, any playlists populated
with such source are expecting this item to run until the end of the schedule. Any other entries
following such item are therfore skipped. This might change in a future version of AURA Tank,
allowing to set a length property.

The switching between types of audio source is handled automatically. To learn more check out the
[Scheduling](### Secure Scheduling) section.

## Multi-channel output

### Analog line-out

In most scenarios it might be sufficient to broadcast via only one analog stereo output.
If needed you can configure up to five stereo pairs.

### Stream to Icecast

Engine allows to stream to multiple Icecast Servers simultaniousely. It is also sending meta information
to the streaming server, using the *Icy* protocol.

To configure your Icecast connectivity check-out the `[stream]` section in your configuration.

### Record output to filesystem

Engine allows recording broadcasts to the filesystem using up to multiple recorders. Each recorder can
be configured to save in a different resolution in audio quality.

## Scheduling

Engine provide a scheduling functionality by polling external API endpoints frequently.

Scheduling is split into multiple phase. Below you see a timline with one schedule planned at a certain
point in time and the involved phase before:

```
================= [   Scheduling Window   ] ============= [        Schedule Play-out                 ] ====

== (FILESYSTEM A) ========================== [ Pre-Roll ] [  Play 4  ] ====================================
== (STREAM A) ========================================== [ Pre-Roll ] [ Play 1 ] ==========================
== (LIVE 1) ====================================================== [ Pre-Roll ] [ Play 1 ] ================
== (FILESYSTEM B) ========================================================== [ Pre-Roll ] [  Play 4  ] ====
```

- **Scheduling Window**: Within the scheduling window any commands for controlling
    the mixer of the soundsystem are prepared and queued.

    Until the start of the window, schedules can be added or removed
    via external API Endpoints (e.g. using Steering or Dashboard). Until here any
    changes on the schedule itself will be reflected in the actual play-out. Be aware,
    that this only affects the schedule ("timeslot") itself, it does not involve related
    playlists and their entries. The latter can still be modified within the scheduling
    window.

    The start and the end of the window is defined by the start of the schedule minus
    a configured amount of seconds (see `scheduling_window_start` and `scheduling_window_end`
    in `engine.ini`).

    During the scheduling window, the external API Endpoints are pulled continiously, to
    check for updated schedules and related playlists. Also, any changes to playlists and
    its entries are respected within that window (see `fetching_frequency` in `engine.ini`).

    <u>Pro-active fallback handling</u> by verifying the existence of playlists is already
    happening at this stage.

    > Important: It's vital that the the scheduling window is wider than the fetching frequency.
    Otherwise one fetch might never hit a scheduling window, hence not being able to schedule stuff.

- **Queuing and Pre-Rolling**: Before any playlist entries of the schedule can be turned into
    sound, they need to be grouped, queued an pre-rolled.

    Within this stage users can neither change playlists nor its entries anymore. <u>Pro-active
    fallback handling</u> will happen though (see next chapter below for an explanation).

    1. First, all entries are aggregated when they hold filesystem entries.
        Given you have a playlist with 10 entries, the first 4 are consisting of files, the next two
        of a a stream and a live source. The last 4 are files again. These entries are now
        aggregated into 4 groups: one for the files, one for the stream, one for the live entry
        and another one for files. For each group a timer for executing the next step is created.

    2. Now, the playlist entries are going to be "pre-rolled". This means that filesystem
        entries are queued and pre-loaded and entries which are based on audio streams are buffered.
        This is required to allow a seamless play-out, when its time to do so (in the next stage).
        Due to their nature, playlist entries which hold live audio sources are not affected by
        this stage at all.

    Set the maximum time reserved for pre-rolling in your configuration (compare `preroll_offset`
    in `engine.ini`).

    > Important: This stage can require up to 4x pre-rolling steps, one for default play-out
    and for each of the 3 fallback types. So the expected time to load some source multiplied
    by 4 should be set as the actual value. Also, the offset should not exceed the time between
    the end of the scheduling-window and the start of the actual schedule playout.

- **Play-out**: Finally the actual play-out is happening. The faders of the virtual mixers are pushed
    all the way up, as soon it's "time to play" for one of the pre-rolled entries.
    Transitions between playlist entries with different types of sources (file, stream and analog
    inputs) are performed automatically. At the end of each schedule the channel is faded-out,
    no matter if the total length of the playlist entries would require a longer timeslot.

    Since each scheduled playlist can consist of multiple entry types such as *file*, *live*,
    and *stream*, the play-out of the schedule is actually a bit more complex. The paragraphs
    below go into more details.

    If for some reason the playout is corrupted, stopped or too silent to make any sense, then
    this <u>triggers a 2nd level fallback using the silence detector</u> (see chapter below).


## Fallback Handling

Engine performs scheduling with some additional handling for extended security, to meet the requirements
of community radios. It tries to pro-actively reacted to scenarios, where the upload of some
pre-recorded show has been forgotten, or some live schedule is not taking place.

Usually in such cases the broadcast might end up with some timeslot filled with silence.
To avoid this, Engine implements multiple levels of fallback handling.

To understand how fallbacks are handled by Engine in detail, it's good to know how the Engine's
scheduling is executed. Read more about the scheduling approach in the chapter above. 

The available fallbacks are evaluated in following order:

1. **Schedule Fallback**: If the show fallback is not assigned, a configured fallback
    playlist for the related timeslot is used.

2. **Show Fallback**: If the schedule for some show has no playlist assigned, the
    playlist assigned as a *show fallback* is used instead. In the dashboard this can
    be done as seen in the screenshot below.

    ![Setting for the Show Fallback in AURA Dashboard](images/dashboard-fallback-setting.png "Show Fallback in Dashboard")

3. **Station Fallback**: If everything goes wrong, meaning all the previous fallback
    playlists are missing or are invalid, the *station fallback* will be triggered. This
    fallback type is specified by a playlist ID set in the Engine's configuration
    (see `scheduling_station_fallback_id` in `engine.ini`)

### Pro-active Fallback Handling (1st Level Fallback)

This type of fallback handling is performed before the actual playout, within the scheduling
window and queuing/pre-rolling stage. Here Engine pro-actively verifies the validity of the playlists.
For example it checks if some playlist ist existing at all, if it holds entries or if the
entries are existing.

Furthermore, it checks for any unexpected error when entries are pre-rolled. For example Engine
tries to buffer some HTTP Audio Stream, but it is not reachable. In this case this will
trigger the scheduling of some fallback playlist.

Pro-active fallback handling has the advantage that any (even short) periods of silence
during playlist can be avoided. Still, there are cases, where any failure cannot be detected
beforehand. Therefore a 2nd level fallback solution is required (see below).

### Fallback Handling using the Silence Detector (2nd Level Fallback)

Engine offers a simple way to detect situations where no sound is on air, the so-called
Silence Detector. It allows detection of absoulte silence, weak signals or even noise.

To configure the sensitivity of the Silence Detector check-out following properties in
`engine.ini`:

```
# max_blank => maximum time of blank from source (defaults to 20., seconds, float)
# min_noise => minimum duration of noise on source to switch back over (defaults to 0, seconds, float)
# threshold => power in dB under which the stream is considered silent (defaults to -40., float)
fallback_max_blank="20."
fallback_min_noise="0."
fallback_threshold="-50."
```

## Monitoring

You have following options to monitor the Engine:

* Send mails on errors and warnings
* Engine Status Information
* Engine Heartbeat
* Logging

### Send mails on errors and warnings

To activate you'll need to set some mail account within the `[monitoring]` section of your configuration.

```ini
[monitoring]
mail_server="mail.o94.at"
mail_server_port="587"
mail_user="aura@o94.at"
mail_pass="---SECRET--PASSWORD---"

# If you want to send multiple adminmails, make them space separated
admin_mail="admin-email@your.domain"

# Which from mailadress should be used
from_mail="monitoring@aura.engine"

# The beginning of the subject. With that you can easily apply filter rules using a mail client
mailsubject_prefix="[Aura Engine]"
```

### Engine Health Information via Engine API

Whenever the Engine's status turns into some unhealthy state this is logged to [Engine API](https://gitlab.servus.at/aura/engine-api).
Also, when it returns to some valid state this is logged to the Engine API.

### Engine Heartbeat

Instead of checking all status properties, the Heartbeat only validates the vital ones
required to run the engine. If all of those are valid, as network socket request is sent
to a defined server.

This heartbeat is sent continiously based on the configured `heartbeat_frequency`.

```ini
# Server where heartbeat info is sent to
heartbeat_server = "127.0.0.1"
# Some UDP port
heartbeat_port = 43334
# Seconds how often the vitality of the Engine should be checked (0 = disabled)
heartbeat_frequency = 1
```

The service receiving this heartbeat ticks can decide what to do with that information.
One scenario could be switching to another Engine instance or any other failover scenario.
Under `contrib/heartbeat-monitor` you'll find some sample application digesting these
heartbeat signals.

### Logging

In development all Engine logs can be found under `./log`, and for production they can
are located in `/var/log/aura`. Adapt the log-level within your configuration to get
more or less verbose log output:

```ini
logdir="/var/log/aura"

# Possible values: debug, info, warning, error, critical
loglevel="info"
```

The log directory holds individual logs from Engine Core, Liquidsoap and the API.
But also `stout` outputs from supervisor services are written there.

Additionally you'll finde Supervisor specific logs under`/var/log/supervisor`.



## Read more

- [Overview](/README.md)
- [Installation for Development](installation-development.md)
- [Installation for Production](installation-production.md)
- [Running with Docker](running-docker.md)
- [Setup the Audio Store](docs/setup-audio-store.md)
- [Developer Guide](developer-guide.md)
- [Engine Features](engine-features.md)
- [Frequently Asked Questions (FAQ)](docs/frequently-asked-questions.md)